@@ -1,6 +1,6 @@
 /* Interface to Perl from OCaml.
  * Copyright (C) 2003 Merjis Ltd.
- * $Id: perl_c.c,v 1.19 2004/12/11 15:40:09 rich Exp $
+ * $Id: perl_c.c,v 1.22 2005/02/13 16:33:27 rich Exp $
  */
 
 #include <stdio.h>
@@ -59,6 +59,8 @@ static value Val_xv (SV *sv);
 #define Av_val(avv) ((AV *) Xv_val (avv))
 #define Val_hv(hv) (Val_xv ((SV *)(hv)))
 #define Hv_val(hvv) ((HV *) Xv_val (hvv))
+#define Val_he(he) (Val_voidptr ((he)))
+#define He_val(hev) (Voidptr_val (HE, (hev)))
 
 static void
 xs_init (pTHX)
@@ -127,8 +129,8 @@ perl4caml_string_of_sv (value svv)
   STRLEN len;
   CAMLlocal1 (strv);
   str = SvPV (sv, len);
-  /* XXX This won't work if the string contains NUL. */
-  strv = caml_copy_string (str);
+  strv = caml_alloc_string (len);
+  memcpy (String_val (strv), str, len);
   CAMLreturn (strv);
 }
 
@@ -199,6 +201,30 @@ perl4caml_sv_type (value svv)
     }
 }
 
+CAMLprim value
+perl4caml_address_of_sv (value svv)
+{
+  CAMLparam1 (svv);
+  SV *sv = Sv_val (svv);
+  CAMLreturn (caml_copy_nativeint ((long) sv));
+}
+
+CAMLprim value
+perl4caml_address_of_av (value avv)
+{
+  CAMLparam1 (avv);
+  AV *av = Av_val (avv);
+  CAMLreturn (caml_copy_nativeint ((long) av));
+}
+
+CAMLprim value
+perl4caml_address_of_hv (value hvv)
+{
+  CAMLparam1 (hvv);
+  HV *hv = Hv_val (hvv);
+  CAMLreturn (caml_copy_nativeint ((long) hv));
+}
+
 CAMLprim value
 perl4caml_scalarref (value svv)
 {
@@ -248,7 +274,12 @@ perl4caml_deref (value svv)
   default:
     caml_invalid_argument ("deref: SV is not a reference to a scalar");
   }
-  rsvv = Val_sv (SvRV (sv));
+  sv = SvRV (sv);
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced SV.
+   */
+  sv = SvREFCNT_inc (sv);
+  rsvv = Val_sv (sv);
   CAMLreturn (rsvv);
 }
 
@@ -267,7 +298,12 @@ perl4caml_deref_array (value svv)
   default:
     caml_invalid_argument ("deref_array: SV is not a reference to an array");
   }
-  ravv = Val_av ((AV *) SvRV (sv));
+  sv = SvRV (sv);
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced AV.
+   */
+  sv = SvREFCNT_inc (sv);
+  ravv = Val_av ((AV *) sv);
   CAMLreturn (ravv);
 }
 
@@ -286,7 +322,12 @@ perl4caml_deref_hash (value svv)
   default:
     caml_invalid_argument ("deref_hash: SV is not a reference to a hash");
   }
-  rhvv = Val_hv ((HV *) SvRV (sv));
+  sv = SvRV (sv);
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced HV.
+   */
+  sv = SvREFCNT_inc (sv);
+  rhvv = Val_hv ((HV *) sv);
   CAMLreturn (rhvv);
 }
 
@@ -346,6 +387,10 @@ perl4caml_av_pop (value avv)
   CAMLparam1 (avv);
   AV *av = Av_val (avv);
   SV *sv = av_pop (av);
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced AV.
+   */
+  sv = SvREFCNT_inc (sv);
   CAMLreturn (Val_sv (sv));
 }
 
@@ -368,6 +413,10 @@ perl4caml_av_shift (value avv)
   CAMLparam1 (avv);
   AV *av = Av_val (avv);
   SV *sv = av_shift (av);
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced AV.
+   */
+  sv = SvREFCNT_inc (sv);
   CAMLreturn (Val_sv (sv));
 }
 
@@ -398,6 +447,10 @@ perl4caml_av_get (value avv, value i)
   AV *av = Av_val (avv);
   SV **svp = av_fetch (av, Int_val (i), 0);
   if (svp == 0) caml_invalid_argument ("av_get: index out of bounds");
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced AV.
+   */
+  *svp = SvREFCNT_inc (*svp);
   CAMLreturn (Val_sv (*svp));
 }
 
@@ -455,6 +508,10 @@ perl4caml_hv_get (value hvv, value key)
   HV *hv = Hv_val (hvv);
   SV **svp = hv_fetch (hv, String_val (key), caml_string_length (key), 0);
   if (svp == 0) caml_raise_not_found ();
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced SV.
+   */
+  SvREFCNT_inc (*svp);
   CAMLreturn (Val_sv (*svp));
 }
 
@@ -494,6 +551,73 @@ perl4caml_hv_undef (value hvv)
   CAMLreturn (Val_unit);
 }
 
+CAMLprim value
+perl4caml_hv_iterinit (value hvv)
+{
+  CAMLparam1 (hvv);
+  HV *hv = Hv_val (hvv);
+  int i = hv_iterinit (hv);
+  CAMLreturn (caml_copy_int32 (i));
+}
+
+CAMLprim value
+perl4caml_hv_iternext (value hvv)
+{
+  CAMLparam1 (hvv);
+  CAMLlocal1 (hev);
+  HV *hv = Hv_val (hvv);
+  HE *he = hv_iternext (hv);
+  if (he == NULL) caml_raise_not_found ();
+  hev = Val_he (he);
+  CAMLreturn (hev);
+}
+
+CAMLprim value
+perl4caml_hv_iterkey (value hev)
+{
+  CAMLparam1 (hev);
+  CAMLlocal1 (strv);
+  HE *he = He_val (hev);
+  I32 len;
+  char *str = hv_iterkey (he, &len);
+  strv = caml_alloc_string (len);
+  memcpy (String_val (strv), str, len);
+  CAMLreturn (strv);
+}
+
+CAMLprim value
+perl4caml_hv_iterval (value hvv, value hev)
+{
+  CAMLparam2 (hvv, hev);
+  CAMLlocal1 (svv);
+  HV *hv = Hv_val (hvv);
+  HE *he = He_val (hev);
+  SV *sv = hv_iterval (hv, he);
+  SvREFCNT_inc (sv);
+  svv = Val_sv (sv);
+  CAMLreturn (svv);
+}
+
+CAMLprim value
+perl4caml_hv_iternextsv (value hvv)
+{
+  CAMLparam1 (hvv);
+  CAMLlocal3 (strv, svv, rv);
+  HV *hv = Hv_val (hvv);
+  char *str; I32 len;
+  SV *sv = hv_iternextsv (hv, &str, &len);
+  if (sv == NULL) caml_raise_not_found ();
+  SvREFCNT_inc (sv);
+  svv = Val_sv (sv);
+  strv = caml_alloc_string (len);
+  memcpy (String_val (strv), str, len);
+  /* Construct a tuple (strv, svv). */
+  rv = caml_alloc_tuple (2);
+  Field (rv, 0) = strv;
+  Field (rv, 1) = svv;
+  CAMLreturn (rv);
+}
+
 CAMLprim value
 perl4caml_get_sv (value optcreate, value name)
 {
@@ -505,6 +629,11 @@ perl4caml_get_sv (value optcreate, value name)
   sv = get_sv (String_val (name), create == Val_true ? TRUE : FALSE);
   if (sv == NULL) caml_raise_not_found ();
 
+  /* Increment the reference count because we're creating another
+   * value pointing at the referenced SV.
+   */
+  SvREFCNT_inc (sv);
+
   CAMLreturn (Val_sv (sv));
 }
 
@@ -519,6 +648,11 @@ perl4caml_get_av (value optcreate, value name)
   av = get_av (String_val (name), create == Val_true ? TRUE : FALSE);
   if (av == NULL) caml_raise_not_found ();
 
+  /* Increment the reference count because we're creating another
+   * value pointing at the AV.
+   */
+  SvREFCNT_inc (av);
+
   CAMLreturn (Val_av (av));
 }
 
@@ -533,6 +667,11 @@ perl4caml_get_hv (value optcreate, value name)
   hv = get_hv (String_val (name), create == Val_true ? TRUE : FALSE);
   if (hv == NULL) caml_raise_not_found ();
 
+  /* Increment the reference count because we're creating another
+   * value pointing at the HV.
+   */
+  SvREFCNT_inc (hv);
+
   CAMLreturn (Val_hv (hv));
 }
 
@@ -1061,6 +1200,7 @@ Val_voidptr (void *ptr)
 static void
 xv_finalize (value v)
 {
+  /*fprintf (stderr, "about to decrement %p\n", Xv_val (v));*/
   SvREFCNT_dec ((SV *) Xv_val (v));
 }
 
