@@ -2,7 +2,7 @@
   *
   * Copyright (C) 2003 Merjis Ltd.
   *
-  * $Id: pl_HTML_Parser.ml,v 1.3 2003/10/16 13:41:07 rich Exp $
+  * $Id: pl_HTML_Parser.ml,v 1.4 2004/11/23 13:40:50 rich Exp $
   *)
 
 open Perl
@@ -19,7 +19,7 @@ object (self)
     call_method_void sv "parse" [sv_of_string content]
   method eof =
     call_method_void sv "eof" []
-  method delete =
+  method delete () =
     call_method_void sv "delete" []
 
 end
