@@ -2,16 +2,19 @@
   *
   * Copyright (C) 2003 Merjis Ltd.
   *
-  * $Id: pl_HTTP_Message.ml,v 1.3 2003/10/16 13:41:07 rich Exp $
+  * $Id: pl_HTTP_Message.ml,v 1.4 2004/11/22 17:08:36 rich Exp $
   *)
 
 open Perl
 
+open Pl_HTTP_Headers
+
 let _ = eval "use HTTP::Message"
 
 class http_message sv =
 
 object (self)
+  inherit http_headers sv
 
   method content =
     string_of_sv (call_method sv "content" [])
