@@ -2,7 +2,7 @@
   *
   * Copyright (C) 2003 Merjis Ltd.
   *
-  * $Id: pl_HTTP_Request.ml,v 1.3 2003/10/16 13:41:07 rich Exp $
+  * $Id: pl_HTTP_Request.ml,v 1.4 2005/01/13 16:06:04 rich Exp $
   *)
 
 open Perl
@@ -23,6 +23,10 @@ object (self)
     string_of_sv (call_method sv "method" [])
   method set_method meth =
     call_method_void sv "method" [sv_of_string meth]
+  method uri =
+    string_of_sv (call_method sv "uri" [])
+  method set_uri uri =
+    call_method_void sv "uri" [sv_of_string uri]
   method as_string =
     string_of_sv (call_method sv "as_string" [])
 
