@@ -2,12 +2,13 @@
   *
   * Copyright (C) 2003 Merjis Ltd.
   *
-  * $Id: pl_HTTP_Response.ml,v 1.3 2003/10/16 13:41:07 rich Exp $
+  * $Id: pl_HTTP_Response.ml,v 1.4 2005/01/13 16:06:04 rich Exp $
   *)
 
 open Perl
 
 open Pl_HTTP_Message
+open Pl_HTTP_Request
 
 let _ = eval "use HTTP::Response"
 
@@ -24,6 +25,10 @@ object (self)
     string_of_sv (call_method sv "message" [])
   method set_message message =
     call_method_void sv "message" [sv_of_string message]
+  method request =
+    new http_request (call_method sv "request" [])
+  method set_request (req : http_request) =
+    call_method_void sv "request" [req#sv]
   method status_line =
     string_of_sv (call_method sv "status_line" [])
   method base =
