@@ -2,7 +2,7 @@
   *
   * Copyright (C) 2003 Merjis Ltd.
   *
-  * $Id: pl_URI.ml,v 1.3 2003/10/16 13:41:07 rich Exp $
+  * $Id: pl_URI.ml,v 1.5 2005/02/13 17:09:14 rich Exp $
   *)
 
 open Perl
@@ -31,6 +31,8 @@ object (self)
     string_of_sv (call_method sv "fragment" [])
   method set_fragment fragment =
     call_method_void sv "fragment" [sv_of_string fragment]
+  method set_no_fragment () =
+    call_method_void sv "fragment" [sv_undef ()]
   method as_string =
     string_of_sv (call_method sv "as_string" [])
   method canonical =
@@ -40,6 +42,22 @@ object (self)
   method rel base =
     string_of_sv (call_method sv "rel" [sv_of_string base])
 
+
+  method host =
+    string_of_sv (call_method sv "host" [])
+  method set_host host =
+    call_method_void sv "host" [sv_of_string host]
+  method port =
+    string_of_sv (call_method sv "port" [])
+  method set_port port =
+    call_method_void sv "port" [sv_of_string port]
+  method host_port =
+    string_of_sv (call_method sv "host_port" [])
+  method set_host_port host_port =
+    call_method_void sv "host_port" [sv_of_string host_port]
+  method default_port =
+    int_of_sv (call_method sv "default_port" [])
+
 end
 
 let new_ ?scheme str =
@@ -50,3 +68,8 @@ let new_ ?scheme str =
       | Some scheme -> [sv_of_string scheme] in
   let sv = call_class_method "URI" "new" args in
   new uri sv
+
+let new_abs str base =
+  let sv = call_class_method "URI" "new_abs" [sv_of_string str;
+					      sv_of_string base] in
+  new uri sv
